/*
 * copyright (C) 2018 CERN (www.cern.ch)
 * author Federico Vaga <federico.vaga@cern.ch>
 * author Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
 *
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */

#define WRTD_IN_PORT 34700
#define WRTD_OUT_PORT 34701

struct wrtd_trigger_message {
	uint32_t transmit_seconds;
	uint32_t transmit_cycles;
	int count;
	struct wrtd_trigger_entry triggers[TDC_TRIGGER_COALESCE_LIMIT];
	uint32_t pad; // stupid Etherbone for some reasons drops the last entry on TX
};


static inline void ts_add(struct wr_timestamp *a, const struct wr_timestamp *b)
{
	a->frac += b->frac;

	if(a->frac >= 4096)
	{
		a->frac -= 4096;
		a->ticks ++;
	}

	a->ticks += b->ticks;

	if(a->ticks >= 125000000)
	{
		a->ticks -= 125000000;
		a->seconds++;
	}

	a->seconds += b->seconds;
}

static inline void ts_sub(struct wr_timestamp *a, const struct wr_timestamp *b)
{
	a->frac -= b->frac;

	if(a->frac < 0)
	{
		a->frac += 4096;
		a->ticks --;
	}

	a->ticks -= b->ticks;

	if(a->ticks < 0)
	{
		a->ticks += 125000000;
		a->seconds--;
	}

	a->seconds -= b->seconds;

	if(a->seconds == -1)
	{
		a->seconds = 0;
		a->ticks -= 125000000;
	}
}
